/* Copyright (C) 2002 GFRN systems

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   The latest version of this program may be found at
   http://CQiNet.sourceforge.net

   $Log: thebridge.c,v $
   Revision 1.2  2002/08/18 16:38:05  wb6ymh
   Added code to enable core dumps (thanks to WD4NMQ).

   Revision 1.1.1.1  2002/08/10 20:33:41  wb6ymh
   initial import

*/

#include "common.h"

#ifndef _WIN32
   // FreeBSD, Linux, etc..
   #include <stdio.h>
   #include <stdlib.h>
   #include <sys/types.h>
   #ifdef TIME_WITH_SYS_TIME
      #include <sys/time.h>
      #include <time.h>
   #else
      #ifdef HAVE_SYS_TIME_H
         #include <sys/time.h>
      #else
         #include <time.h>
      #endif
   #endif
   #ifdef HAVE_SYS_TIMEB_H
      #include <sys/timeb.h>
   #endif
   #include <unistd.h>
   #include <sys/socket.h>
   #include <netinet/in.h>
   #include <arpa/inet.h>
   #include <errno.h>
   #include <string.h>
#ifdef HAVE_FCNTL_H
   #include <fcntl.h>
#endif
#ifdef HAVE_LIMITS_H
   #include <limits.h>
#else
   #define  INT_MAX     0x7fffffff  /* assume 32 bit ints */
#endif
   #include <syslog.h>
   #include <sys/resource.h>
#else
   // Windoze
   #include <stdio.h>
   #include <time.h>
   #include <sys\timeb.h>
   #include <io.h>
   #include <winsock2.h>
   #include <limits.h>
#endif

#include "avl.h"
#include "main.h"
#include "configvars.h"
#include "conference.h"
#include "dirclient.h"
#include "users.h"

#ifdef _WIN32
// Windoze
   WSADATA wsad;
   #define CONFIG_FILE_PATH   "tbd.conf"
#else
   #define PID_FILE_PATH      "/var/run/tbd.pid"
   #define CONFIG_FILE_PATH   SYSCON_DIR "/tbd.conf"
   #define TEST SYSCON_DIR
#endif

#ifndef HAVE_GETOPT
// globals needed by getopt
int optind = 1;
int optopt;
char *optarg;
int getopt(int argc,char **argv,char *opts);
#endif

char *ConfigFilePath = NULL;

int main(int argc, char *argv[], char *envp[])
{
   int Err = 0;
   char  Option;
   int Slient = 0;
#ifndef _WIN32
   FILE *fp;
   struct rlimit rlim;
#endif

   while((Option = getopt(argc, argv, "df:s")) != -1) {
      switch(Option) {
         case 'd':   // Debug mode
            if(++DebugLevel > 3) {
               printf("Sorry, -d x 3 is as verbose as I get!\n");
               exit(1);
            }
            break;

         case 'f':   // configuration file path
            ConfigFilePath = strdup(optarg);
            break;
         
         case 's':
            Slient++;
            break;

         case '?':
            printf("Usage: tbd [-d] [-f config file path] [-s]\n");
            printf("\t-d - Debug (with increasing verbosity if used "
                   "multiple times)\n");
            printf("\t-f - set configuration file name.\n");
            printf("\t-s - slient - suppress startup messages.\n");
            exit(1);
      }
   }

   if(!Slient) {
      printf(PACKAGE " Version " VERSION "\n");
   }
   
   if(ReadConfig()) {
      return ERR_CONFIG_FILE;
   }
   
   if(strcmp(ConferenceCall,"W1AW") == 0) {
      printf("Come on now, your call isn't really W1AW is it!\n");
      printf("Please edit tbd.conf and try again.\n");
      exit(2);
   }

   ClientTree = avl_create(ClientCompare,NULL,NULL);
   UserIPAdrTree = avl_create(UserIPCompare,NULL,NULL);
   UserTree = avl_create(UserCompare,NULL,NULL);
   
   if(ClientTree == NULL || UserTree == NULL || UserIPAdrTree == NULL) {
   // That ain't good !
      printf("main(): Fatal error, unable to create AVL trees.\n");
      return ERR_CREATE_AVL;
   }


#ifdef _WIN32
   /* Initialize Winsock*/
   WSAStartup(0x0101, &wsad);

   if(!DebugLevel) {
   // Windoze always runs in at least DebugLevel 1 'cuz it doesn't have
   // a daemon mode
      DebugLevel = 1;
   }

#else

// On some systems code dumps are disabled by default, 
// enable them so we can debug this thing !

   rlim.rlim_cur = RLIM_INFINITY;
   if(setrlimit(RLIMIT_CORE, &rlim)) {
      perror("setrlimit");
      exit(1);
   }

   if(!DebugLevel) {
      pid_t pid;

      if(!Slient) {
         printf("Becoming a Daemon ...\n");
      }
      openlog("tbd",LOG_NOWAIT, LOG_LOCAL5);
      bDaemon = TRUE;

      // Fork daemon
      if((pid = fork()) < 0) {
         perror("Forking");
         exit(1);
      }
      else if(pid != 0) {
      // parent
         exit(0);
      }
      
      // must be the child
      LOG_NORM((PACKAGE " V " VERSION " initializing"));
      if(setsid() == -1) {
         LOG_ERROR(("setsid() failed: %m"));
         exit(2);
      }
      if(daemon(1, 0) < 0) {
         LOG_ERROR(("daemon(1,0) failed: %m"));
      }

      if((fp = fopen(PID_FILE_PATH,"w")) != NULL) {
         fprintf(fp,"%d\n",getpid());
         fclose(fp);
      }
      else {
         LOG_ERROR(("Unable to create pid file: %m"));
      }
   }
#endif
   
   if(Err == 0) {
      ConferenceInit();
   }

   if(Err == 0) {
      MainLoop();
   }
   SetConfigDefaults(FALSE);  // free dynamic allocated strings
   return 0;
}

int ReadConfig()
{
   char *ConfigFile = ConfigFilePath;
    
   if(ConfigFile == NULL) {
      ConfigFile = CONFIG_FILE_PATH;
   }
   
   SetConfigDefaults(TRUE);
   return read_config(ConfigFile,&ConfigVars);
}

void Shutdown()
{
   ServerRequest(SERV_REQ_SHUTDOWN);
}


#ifndef HAVE_GETOPT

/*
 * getopt a wonderful little function that handles the command line.
 * available courtesy of AT&T.
 */
int getopt(int argc,char **argv,char *opts)
{
   static int sp = 1;
   register int c;
   register char *cp;

   if(sp == 1)
      if(optind >= argc ||
         argv[optind][0] != '-' || argv[optind][1] == '\0')
         return(EOF);
      else if(strcmp(argv[optind], "--") == 0) {
         optind++;
         return(EOF);
      }
   optopt = c = argv[optind][sp];
   if(c == ':' || (cp=strchr(opts, c)) == NULL) {
      printf("addcr: illegal option -- ", c);
      if(argv[optind][++sp] == '\0') {
         optind++;
         sp = 1;
      }
      return('?');
   }
   if(*++cp == ':') {
      if(argv[optind][sp+1] != '\0')
         optarg = &argv[optind++][sp+1];
      else if(++optind >= argc) {
         printf("addcr: option requires an argument -- ", c);
         sp = 1;
         return('?');
      }
      else
         optarg = argv[optind++];
      sp = 1;
   }
   else {
      if(argv[optind][++sp] == '\0') {
         sp = 1;
         optind++;
      }
      optarg = NULL;
   }
   return(c);
}
#endif


